@@ -10,15 +10,13 @@
  * See the COPYING file in the top-level directory.
  *
  */
+
+#include "qemu/osdep.h"
+#include <glib.h>
+
 #include "libqtest.h"
 #include "hw/timer/mc146818rtc_regs.h"
 
-#include <glib.h>
-#include <stdio.h>
-#include <string.h>
-#include <stdlib.h>
-#include <unistd.h>
-
 static uint8_t base = 0x70;
 
 static int bcd2dec(int value)
@@ -552,7 +550,7 @@ int main(int argc, char **argv)
 
     g_test_init(&argc, &argv, NULL);
 
-    s = qtest_start("-display none -rtc clock=vm");
+    s = qtest_start("-rtc clock=vm");
     qtest_irq_intercept_in(s, "ioapic");
 
     qtest_add_func("/rtc/check-time/bcd", bcd_check_time);
